/* wptMDSumDlg.cpp
 *	Copyright (C) 2003, 2005, 2006 Timo Schulz
 *
 * This file is part of WinPT.
 *
 * WinPT is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * WinPT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with WinPT; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <windows.h>
#include <stdio.h>

#include "resource.h"
#include "wptTypes.h"
#include "wptW32API.h"
#include "wptGPG.h"
#include "wptCommonCtl.h"
#include "wptContext.h"
#include "wptNLS.h"
#include "wptErrors.h"


/* Return a printable digest of the buffer @mdbuf. */
static const char*
printable_digest (byte *mdbuf, size_t n)
{
    static char mdasc[64];
    size_t i;
    for (i = 0; i < n; i++)
	sprintf (mdasc+2*i, "%02X", mdbuf[i]);
    return mdasc;
}


static const char*
id2algo (int mdalgo)
{
    gpgme_hash_algo_t md = (gpgme_hash_algo_t)mdalgo;

    switch (md) {
    case GPGME_MD_MD5:	    return "MD5";
    case GPGME_MD_SHA1:	    return "SHA1";
    case GPGME_MD_RMD160:   return "RMD160";
    case GPGME_MD_SHA256:   return "SHA256";
    default: return "";
    }
    return "";
}


/* Dialog box procedure to show and calculate file digests. */
BOOL CALLBACK
mdsum_dlg_proc (HWND dlg, UINT msg, WPARAM wparam, LPARAM lparam)
{
    static listview_ctrl_t lv;
    static struct md_file_s * md;
    struct listview_column_s cols[] = {
        {0, 264, (char *)_("Digest")},
        {1, 128, (char *)_("Name")},
        {0, 0, 0}
    };
    gpgme_data_t sumlist;
    const char *name;
    char fname[300], mdasc[64];
    byte mdbuf[20];
    int i;
    size_t n;

    switch( msg ) {
    case WM_INITDIALOG:
	md = (md_file_s *)lparam;
	if( !md )
	    BUG( NULL );	
	if( listview_new( &lv ) )
	    BUG( NULL );
	lv->ctrl = GetDlgItem( dlg, IDC_MDSUM_LIST );
	for( i = 0; i < cols[i].width; i++ )
	    listview_add_column( lv, &cols[i] );
	for( i = 0; i < listview_count_items( md->lv, 0 ); i++ ) {
	    if( listview_get_item_state( md->lv, i ) ) {
		listview_get_item_text( md->lv, i, 1, fname, sizeof (fname)-1 );
		if( !gpg_md_hash_file( md->mdalgo, fname, mdbuf, &n ) ) {
		    listview_add_item( lv, "" );
		    listview_add_sub_item( lv, 0, 0, printable_digest( mdbuf, n ) );
		    listview_add_sub_item( lv, 0, 1, fname );
		}
	    }
	}
	SetDlgItemText (dlg, IDC_MDSUM_COPY, _("&Save..."));
	SetDlgItemText (dlg, IDOK, _("&Close"));
	SetDlgItemText (dlg, IDC_MDSUM_TOCLIP, _("Save to clipboard"));
	SetWindowText (dlg, _("Print Message Digest"));
	SetForegroundWindow (dlg);
	break;

    case WM_SYSCOMMAND:
	if (wparam == SC_CLOSE)
	    EndDialog (dlg, TRUE);
	return TRUE;

    case WM_DESTROY:
	if( lv ) {
	    listview_release( lv );
	    lv = NULL;
	}
	break;

    case WM_COMMAND:
	switch (LOWORD (wparam)) {
	case IDOK:
	    EndDialog (dlg, TRUE);
	    break;

	case IDC_MDSUM_COPY:
	    if (gpgme_data_new (&sumlist))
		BUG(0);
	    if (md->mdalgo == (int)GPGME_MD_RMD160) {
		const char *s = "# warning rmd160sum is not yet available\r\n";
		gpgme_data_write (sumlist, s, strlen (s));
	    }
	    for (i = 0; i < listview_count_items (lv, 0); i++) {
		listview_get_item_text (lv, i, 0, mdasc, DIM (mdasc)-1);
		listview_get_item_text (lv, i, 1, fname, DIM (fname)-1);
		
		gpgme_data_write (sumlist, mdasc, strlen (mdasc));
		gpgme_data_write (sumlist, " ", 1);
		gpgme_data_write (sumlist, fname, strlen (fname));
		gpgme_data_write (sumlist, "\r\n", 2);
	    }
	    if (IsDlgButtonChecked (dlg, IDC_MDSUM_TOCLIP)) {
		gpg_data_release_and_set_clipboard (sumlist, 0);
		break;
	    }
	    _snprintf (fname, sizeof (fname)-1, "%s_sums.txt", id2algo (md->mdalgo));
	    name = get_filesave_dlg (dlg, _("Select file to save checksums"), 
				     NULL, fname);
	    if (name && *name) {
		gpg_data_release_and_set_file (sumlist, name);
		log_box (_("File Manager"), MB_OK, 
			 _("Checksums successfully saved in '%s'"), name);
	    }
	    else
		gpgme_data_release (sumlist);
	    break;
	}
	break;
    }
    return FALSE;
}
